 Program ToCompareCatalogues ;
 Uses
   UnConTyp ,
   UnForFun ,
   Un2000_C , UnQM2000 , Un2000_S ,
   ZUnCoTy  , ZUniTime , ZUnSiMat ;
 Type
   { for only bright stars catalogue }
   TRecStar        =  Record
                        id           : LongInt ;
                        FK5num       : Word ;
                        ra   , sd    : LongInt ; { alfa southdelta in mas }
                        pma  , pmd   : LongInt ; { proper motion 0.01 mas/Year }
                        bt   , vt    : Integer ; { B V photometry 0.001 m }
                      End;
 Var
   PrtFile  : Text ;
   CatFile  : Text ;
   RecFile  : File Of TRecStar ;
   RecStar  : TRecStar ;
   numstar  : LongInt ;
   alphac   : Extended ;
   deltac   : Extended ;
   alphah   : Extended ;
   deltah   : Extended ;
   alphaf   : Extended ;
   deltaf   : Extended ; 
   valphah  : Extended ;
   vdeltah  : Extended ;
   valphaf  : Extended ;
   vdeltaf  : Extended ;
   date0    : Extended ;
   datec    : Extended ;
   dmjd0    : Extended ;
   dmjdc    : Extended ;
   rocur    : Extended ;
   vh,vf    : Extended ;
   ua,ud    : Extended ;
   dalpha   : Array [1..1567] of Single ;
   ddelta   : Array [1..1567] of Single ;
   boostar  : Boolean ;
 Var
   npmatr,rotm,rmatr  : TypeDim33 ;
   cmatr,hmatr,fmatr  : TMatr33 ;
   vecth,vectf,vectc  : TVect3 ;
 Procedure ToTranspoMatr ( a  : TMatr33 ;  Var  b  : TMatr33 ) ;
 Var
   i,j  : Byte ;
 Begin
   For i:=1 To 3 Do
     For j:=1 To 3 Do
       b[j,i]:=a[i,j];
 End;
 Procedure ToOtherMatrType ( a  : TypeDim33 ;  Var  b  : TMatr33 ) ;
 Var
   i,j  : Byte ;
 Begin
   For i:=1 To 3 Do
     For j:=1 To 3 Do  b[i,j]:=a[i,j];
 End;
 Procedure SpherCoorToVect ( Date    : Extended ;
                             Alpha   : Extended ;
                             Delta   : Extended ;
                             VAlpha  : Extended ;
                             VDelta  : Extended ;
                             Var  PosFix  : TVect3 ) ;
 Var
   i,j,k            : Byte ;
   a,d,dt,rr,
   sa,ca,sd,cd,
   va,vd,vx,vy,vz,
   x,y,z,xx,yy,zz   : Extended ;
 Begin
   dt:=(Date-JD2000)/365.25; { differences in year from J2000.0}
   a:=Alpha; { in radian }
   d:=Delta;
   va:=VAlpha; { in radian }
   vd:=VDelta;
   sa:=Sin(a);
   ca:=Cos(a);
   sd:=Sin(d);
   cd:=Cos(d);
   x:=ca*cd; { position at J2000.0 }
   y:=sa*cd;
   z:=sd;
   vx:=-va*sa-vd*sd*ca;
   vy:=+va*ca-vd*sd*sa;
   vz:=+vd*cd;
   xx:=x+vx*dt; { new position at Date but in J2000.0}
   yy:=y+vy*dt;
   zz:=z+vz*dt;
   rr:=Sqrt(xx*xx+yy*yy+zz*zz);
   PosFix[1]:=xx/rr;
   PosFix[2]:=yy/rr;
   PosFix[3]:=zz/rr;
 End;
 Procedure ClcSpherCoorInDegree ( Pos : TVect3 ;
                                  Var Alfa , Delta, Ro : Extended );
 Var
   xr,xv : TVect3;
   r,rr,q,qq,p,f,u,v,vr,s,c,a,b,d,au : Extended;
   i : integer;
 Begin
   for i:=1 to 3 do  xr[i]:=Pos[i];
   qq:=Sqr(xr[1])+Sqr(xr[2]); rr:=qq+Sqr(xr[3]); r:=Sqrt(rr);
   q:=Sqrt(qq); p:=1/q; f:=1/r;
   c:=p*xr[1]; s:=p*xr[2];    ArcTanTwoArg(s,c,a);
   c:=q*f;     s:=xr[3]*f;    ArcTanTwoArg(s,c,b);
   if  a < 0  then  a:=a+Pi22;
   Alfa:=RadGra*a;
   Delta:=RadGra*b;
   Ro:=r;
 End;
 Procedure ToReadCatFile ( num  : Word ;
                           Var  a , d , va , vd , vf  : Extended ) ;
 Var
   ns  : Word ;
   stemp  : String[2] ;
   ah,am,dd,dm  : Integer ;
   chd,chpa,chpd  : Char ;
   as,pa,ds,pd,v  : Extended ;
 Begin
   boostar:=FALSE;
   ReSet(CatFile);
   While  NOT EOF(CatFile)  Do
     Begin
       ReadLn(CatFile,ns,ah,am,as,stemp,chpa,pa,
                      stemp,chd,dd,dm,ds,stemp,chpd,pd,v);
       If  ns = num
         Then
           Begin
             a:=GraRad*15*(ah+(am+as/60.0)/60.0); { in radian }
             d:=GraRad*(dd+(dm+ds/60.0)/60.0); { in radian }
             If  chd = '-'  Then  d:=-d;
             va:=SecRad*15.0*pa/100.0*Cos(d); { radian/year }
             If  chpa = '-'  Then  va:=-va;
             vd:=SecRad*pd/100.0; { radian/year }
             If  chpd = '-'  Then  vd:=-vd;
             vf:=v;
             boostar:=TRUE;
             Exit;
          End;
     End;
 End;
 Begin
   Assign(PrtFile,'compcat3.prt');
   ReWrite(PrtFile);
   Assign(CatFile,'starsky.cat'); { FK5 catalogue }
   ReSet(CatFile);
   date0:=JD2000;
   datec:=date0+100*365.25;
   dmjd0:=date0-DifEpoch;
   dmjdc:=datec-DifEpoch;
   ClcRotMatr(dmjd0,dmjdc,npmatr,rotm); { IAU 1976 resolution }
   MatrMultMatr(rotm,npmatr,rmatr); { instanteneous equator GreenWich }
   ToOtherMatrType(rmatr,fmatr); { for FK5 stars }
   QRMatr2000(datec,datec,0.0,0.0,cmatr); { IAU 2000 resolution Q(t)*R(-theta) }
   ToTranspoMatr(cmatr,hmatr); { intermediate equator nullo meridian }
   Assign(RecFile,'onlybrs.dat');
   ReSet(RecFile);
   numstar:=0;
   While  NOT EOF(RecFile)  Do
     Begin
       Read(RecFile,RecStar);
       If ( RecStar.FK5num > 0 ) AND ( RecStar.id > 0 )
         Then
           Begin
             With  RecStar  Do
               Begin { Hipparcos Stars }
                 alphah:=GraRad*(ra/3600.00000e3); { in radian }
                 deltah:=GraRad*(sd/3600.00000e3-90); { in radian }
                 valphah:=1.0e-5*SecRad*pma; { in radian/year }
                 vdeltah:=1.0e-5*SecRad*pmd; { in radian/year }
                 vh:=1.0e-3*vt;
               End;
             SpherCoorToVect(datec,alphah,deltah,valphah,vdeltah,vectc);
             MultMatrVec(hmatr,vectc,vecth); { IAU 2000 }
             ClcSpherCoorInDegree(vecth,alphah,deltah,rocur);
             ToReadCatFile(RecStar.FK5num,
                           alphaf,deltaf,valphaf,vdeltaf,vf);
             If  boostar
               Then
                 Begin
                   SpherCoorToVect(datec,
                                   alphaf,deltaf,valphaf,vdeltaf,vectc);
                   MultMatrVec(fmatr,vectc,vectf); { IAU 2000 }
                   ClcSpherCoorInDegree(vectf,alphaf,deltaf,rocur);
                   ua:=3600.0*(alphaf-alphah)
                                          *Cos(GraRad*deltah); { in arcsec }
                   ud:=3600.0*(deltaf-deltah); { in arcsec }
                   If  ( Abs(ua) < 2.0 ) AND ( Abs(ud) < 2.0 )
                         AND ( vh > 4.95 )
                     Then
                       Begin
                         numstar:=numstar+1;
                         dalpha[numstar]:=ua;
                         ddelta[numstar]:=ud;
WriteLn(numstar:5,RecStar.FK5num:6,
        alphah:12:6,deltah:12:6,
        dalpha[numstar]:10:3,ddelta[numstar]:8:3,
        vf:10:3,vh:8:3);
WriteLn(PrtFile,numstar:5,RecStar.FK5num:6,
                alphah:12:6,deltah:12:6,
                dalpha[numstar]:10:3,ddelta[numstar]:8:3,
                vf:10:3,vh:8:3);
                       End;
                 End;
           End;
     End;
   Close(RecFile);
   Close(CatFile);
   Close(PrtFile);
   WriteLn('there is',numstar:10,'  stars from FK5');
 End.