 { simple program to do Chebyshev approximation
   of the precession-nutation parameters
      psi epsilon X Y and value of s(t) }
 Program ApproxNutationParameters ;
 Uses
   UNConTyp , { types, constants }
   UN2000_L , { nutation luni-solar terms }
   UN2000_P , { nutation planetary  terms }
   UNXY2000 , { the value of  X , Y  parameters }
   Un2000_s ; { for function s(t) CEO position parameter }
 Const
   InterPolM  = 11 ;
   InterPoint = InterPolM+2 ;
   NumOfParm  = 5 ;
   StepApp    = 4.0 ; { in day }
 Type
   TOneApp    = Array[1..InterPolM+1] of Double ;
   TApprox    = Record
                  BegApp  : Double ;
                  EndApp  : Double ;
                  PosApp  : Array [1..NumOfParm] of TOneApp ;
                End;
 Var
   polc    : Array [1..InterPolM+1] of Extended ;
   xuzpo   : Array [1..InterPoint] Of Extended ;
   polcpo  : Array [1..InterPolM+1,1..InterPoint] of Extended ;
 Var
   AppFile      : File of TApprox ;
   ApproxC      : TApprox ;
   dpsi,deps    : Extended ; { full nutation }
   dpsil,depsl  : Extended ; { luni-solar nutation }
   dpsip,depsp  : Extended ; { planetary nutation }
   XParm,YParm  : Extended ;
   ParmCur      : Array [1..NumOfParm] of Extended ;
   VectorXYZ    : TVect3 ;   { array[1..3] , type in Un2000_C }
   date1,date2  : Extended ;
   TBeg,TEnd    : Extended ;
   NApproxRec   : LongInt ;

 { to calculate Chebyshev polinoms }

 Procedure ClcChebPol ( n  : Byte ; x  : Extended ) ;
 Var
   i    : Byte ;
 Begin
   polc[1]:=1;
   polc[2]:=x;
   For i:=2 To n Do
     polc[i+1]:=2*x*polc[i]-polc[i-1];
 End;

 { to calculate the values of the Chebyshev polinoms
      in the selected points }

 Procedure ClcChebPolinPoints ;
 Var
   i,j,k,n            : Byte;
   prnpo,xuz          : Extended;
 Begin
   n:=InterpolM;
   prnpo:=0.5e0*2*Pi/(n+1.0e0);
   For k:=1 To InterpolM+1 Do
     Begin
       polc[k]:=0;
       For i:=1 To InterPoint Do
         Begin
           If  k = 1  Then  xuzpo[i]:=0;
           polcpo[k,i]:=0;
         End;
     End;
   For i:=1 To InterPoint Do
     Begin
       j:=InterPoint-i;
       xuz:=Cos(j*prnpo);
       ClcChebPol(n,xuz);
       For k:=1 To InterpolM+1 Do  polcpo[k,i]:=polc[k];
       xuzpo[i]:=xuz;
     End;
 End;

 Procedure ToMakeAppXYS ( Var  TBeg  : Extended ;
                         Var  TEnd  : Extended ) ;
 Var
   i,j,k,ic,jc              : Byte;
   DStep,RatioQ,
   sx,tm,xuz,tuz,q          : Extended ;
   CharTemp                 : Char ;
 Begin
   RatioQ:=1.0e0/(InterpolM+1.0e0);
   For j:=1 To InterpolM+1 Do
     For i:=1 To NumOfParm Do
       ApproxC.PosApp[i][j]:=0;
   ApproxC.BegApp:=TBeg;
   ApproxC.EndApp:=TBeg+StepApp;
   sx:=0.5*(ApproxC.BegApp+ApproxC.EndApp);
   tm:=0.5*(ApproxC.EndApp-ApproxC.BegApp);
   For i:=1 To InterPoint Do
     Begin
       xuz:=xuzpo[i];
       tuz:=sx+tm*xuz;
       NutParmLuSo(tuz,dpsil,depsl); { from Un2000_L in ArcSec }
       NutParmPlan(tuz,dpsip,depsp); { from Un2000_P in ArcSec }
       dpsi:=dpsil+dpsip; { nutation in longitude ArcSec }
       deps:=depsl+depsp; { nutation in obliquity ArcSec }
       ForPNPoleXY(tuz,VectorXYZ); { unit vector of the pole in radian }
       XParm:=RadSec*VectorXYZ[1]; { X from radian to ArcSec }
       YParm:=RadSec*VectorXYZ[2]; { Y from radian to ArcSec }
       For k:=1 To InterpolM+1 Do  polc[k]:=polcpo[k,i];
       ParmCur[1]:=dpsi;
       ParmCur[2]:=deps;
       ParmCur[3]:=XParm;
       ParmCur[4]:=YParm;
       ParmCur[5]:=ToGetCeoPos(tuz,VectorXYZ[1],VectorXYZ[2]); { Un2000_s }
       q:=2;  If  ( ( i = 1 ) OR ( i = InterPoint ) )  Then  q:=1;
       For ic:=1 To InterpolM+1 Do
         For jc:=1 To NumOfParm Do
           ApproxC.PosApp[jc][ic]:=ApproxC.PosApp[jc][ic]
                                  +q*ParmCur[jc]*polc[ic];
     End;
   For ic:=1 To InterpolM+1 Do
     Begin
       q:=RatioQ;  If  ic = 1  Then  q:=0.5*q;
       For jc:=1 To NumOfParm Do
         ApproxC.PosApp[jc][ic]:=q*ApproxC.PosApp[jc][ic];
     End;
   Write(AppFile,ApproxC);
   NApproxRec:=NApproxRec+1;
   TBeg:=ApproxC.EndApp;
   TEnd:=TBeg+StepApp;
 End;

 Begin { main approximation }
   ClcChebPolinPoints ; { to prepare some arrays }
   Assign(AppFile,'parmaps.pat');
   ReWrite(AppFile);
   date1:=JD2000-20*365.25-40.0;
   date2:=JD2000+20*365.25+40.0;
   TBeg:=date1;
   While  TBeg < date2  Do
     Begin
       ToMakeAppXYS(TBeg,TEnd);
       WriteLn(NApproxRec:6,TBeg:15:2,TEnd:15:2);
     End;
   Close(AppFile);
 End.