
 { in units
   Un2000_C  are simple procedures for arguments
   Un2000_L  is procedure to calculate the Luni-Solar nutation
   Un2000_P  is procedure to calculate the planetary terms.
   UnPN2000  are the procedures to obtain matrixes.
   a try to calculate with MHB2000A precession/nutation theory
   Resolution B1.6
   of General Assembly of the International Astronomical Union
   from  IERS Conventions 2000 , web site
   http:\\maia.usno.navy.mil\conv2000.html }

 UNIT Un2000_C ;

 INTERFACE

 Uses
   UnConTyp ; { for type TFundArg , TMatr33 some constants }

 Procedure FundArgLuSo  ( DateJulian  : Extended ;
                          Var  Argum  : TFundArg ) ;

 Procedure FundArgPlan  ( DateJulian  : Extended ;
                          Var  Argum  : TFundArg ) ;

 IMPLEMENTATION


 { FundArgLuSo is a procedure
   to calculate Fundamental Arguments in radians
      on moment DateJulian in Julian Date.
      FundArg[1..5] is massive of the values of arguments.
      1 -> l   Mean Anomaly of the Moon
      2 -> l'  Mean Anomaly of the Sun
      3 -> F   Mean Argument of the latitude of the Moon
      4 -> D   Mean Elongation of the Moon from the Sun
      5 -> Om  Mean Longitude of the Ascending Node of the Moon
       L   Mean Longitude of the Moon
           L=F+Om  or  F=L-Om  or  Om=L-F
      global const - GraRad - for change from degrees to radians. }

 Procedure FundArgLuSo  ( DateJulian  : Extended ;
                          Var  Argum  : TFundArg ) ;
 Const
   fc  : Array [1..5,1..5] of Extended = { polinomial part for  l l' F D Om }
   ( ( +134.96340251, +1717915923.2178, +31.8792, +0.051635, -0.00024470 ) ,
     ( +357.52910918,  +129596581.0481,  -0.5532, +0.000136, -0.00001149 ) ,
     (  +93.27209062, +1739527262.8478, -12.7512, -0.001037, +0.00000417 ) ,
     ( +297.85019547, +1602961601.2090,  -6.3706, +0.006593, -0.00003169 ) ,
     ( +125.04455501,    -6962890.5431,  +7.4722, +0.007702, -0.00005939 ) ) ;
 Var
   i  : Byte ;
   t  : Extended ;
 Begin
   t:=(DateJulian-JD2000)/JulianC; { from the fundamental epoch in century }
   For i:=1 To 5 Do
     Argum[i]:=+GraRad*fc[i][1]
               +SecRad*t*(fc[i][2]+t*(fc[i][3]+t*(fc[i][4]+t*fc[i][5])));
 End;

 { to calculate fundamental arguments of the planet theory in radian }

 Procedure FundArgPlan  ( DateJulian  : Extended ;
                          Var  Argum  : TFundArg ) ;
 Var
   t  : Extended ;
 Begin
   t:=(DateJulian-JD2000)/JulianC; { from the fundamental epoch in century }
   Argum[6] := 4.402608842 + 2608.7903141574 * t; { l the Mercury }
   Argum[7] := 3.176146697 + 1021.3285546211 * t; { l the Venus }
   Argum[8] := 1.753470314 +  628.3075849991 * t; { l the Earth }
   Argum[9] := 6.203480913 +  334.0612426700 * t; { l the Mars }
   Argum[10]:= 0.599546497 +   52.9690962641 * t; { l the Jupiter }
   Argum[11]:= 0.874016757 +   21.3299104960 * t; { l the Saturn }
   Argum[12]:= 5.481293872 +    7.4781598567 * t; { l the Uranus }
   Argum[13]:= 5.311886287 +    3.8133035638 * t; { l the Neptune }
   Argum[14]:= t * (0.024381750+0.00000538691*t); { pa precession rate }
 End;

 END.
