
 { some procedures to use
   binary extended form of DE405/LE405 ephemerides }

 UNIT UnEph405 ; { ephemeris Jet Propulsion Laboratory }

 INTERFACE

 Uses
   UnConTyp; { for Types and Constants }

 Const
   NumAppr405   = 13 ; { number of position in one record }
   NumPlanets   = 11 ;

 Const
   EMRat405         = 81.30056 ; { DE405/LE405 }
   MassR405         = 1.0/EMRat405 ; { DE405/LE405 }
   CoefMassR405     = MassR405/(1+MassR405) ;
   ValueMass405     : Array[1..NumPlanets] of Extended =
                    (  6023600.000,
                        408523.710,
                        332946.050894783312,  { the Earth DE405 }
                       3098708.000,
                          1047.3486000,       { the Jupiter DE405 }
                          3497.8980000,
                         22902.980000,
                         19412.240000,
                     135200000.000,
                      27068700.3875343858,    { the Moon LE405 }
                             1.0    ) ;       { the Sun }

 Const
   NumOrd405  : Array[1..NumAppr405] of Word =
   (  3, 171, 231, 309, 342, 366, 387, 405, 423, 441, 753, 819, 899 ) ;
   NumCof405  : Array[1..NumAppr405] of Byte =
   ( 14,  10,  13,  11,   8,   7,   6,   6,   6,  13,  11,  10,  10 ) ;
   NumDay405  : Array[1..NumAppr405] of Byte =
   (  8,  16,  16,  32,  32,  32,  32,  32,  32,   4,  16,   8,   8 ) ;
   NumCor405  : Array[1..NumAppr405] of Byte =
   (  3,   3,   3,   3,   3,   3,   3,   3,   3,   3,   3,   2,   3 ) ;

 Const
   NMercury  =  1 ;
   NVenus    =  2 ;
   NEarth    =  3 ;
   NMars     =  4 ;
   NJupiter  =  5 ;
   NSaturn   =  6 ;
   NUranus   =  7 ;
   NNeptune  =  8 ;
   NPluto    =  9 ;
   NMoon     = 10 ;
   NSun      = 11 ;
   NNutat    = 12 ;
   NLibrat   = 13 ;

 Type
   TCoef405  = Array [1..1018] of Extended ;

 Var
   CoeFile405         : File of TCoef405;
   CoefEph405         : TCoef405;
   ZStep405           : Extended ;
   ZInit405           : Extended ;
   ZLast405           : Extended ;
   NRec405,MRec405    : LongInt;
   FMPlanet,FMPlaner  : Array [1..NumPlanets] of Extended ;

 Const 
   BooExistEph405     : Boolean = FALSE ;
   BooExistPos405     : Boolean = FALSE ;

 Procedure PosVel405 ( NPlanet  : Byte ;
                       TinTDB   : Extended ;
                       Var  Pos , Vel  : TVect3 ) ;

 Procedure FactMu405 ; { to (km**3/s**2)/c**2 c=VelOfLight }

 Procedure ToGetF405 ; { all variables are external }

 Procedure ToRead405 ( TinTDB  : Extended ) ;

 Procedure EarthB405 ( TinTDB : Extended ;
                       Var  PosEarth , VelEarth : TVect3 ) ;

 Procedure MoonB405 ( TinTDB : Extended ;
                      Var  PosMoon , VelMoon : TVect3 ) ;

 Procedure EarthMoonB405 ( TinTDB  : Extended ;
                           Var  PosEarth , VelEarth  : TVect3 ;
                           Var  PosMoon  , VelMoon   : TVect3 ) ;

 Procedure AccEarth405 ( t  : Extended ;  Var  a  : TVect3 ) ;

 IMPLEMENTATION

 { PosVel405 is a procedure
   to obtain the position of the Planet
      Position in km Velosity in km per second
      reffered to the barycentre from DE405/LE405 Ephemeris }

 Procedure PosVel405 ( NPlanet  : Byte ;
                       TinTDB   : Extended ;
                       Var  Pos , Vel  : TVect3 ) ;
 Var
   PolQuanT,PolQuanU : array[0..15] of Extended;
   CoefQuant : array[1..3,0..15] of Extended;
   Tn,Tk,Tnn,Tkk,t,r,w,rw,x,xw,xr,xv : Extended;
   i,i1,i2,j,k,l,l0,m,n,np : integer;
 Begin
   ToRead405(TinTDB);
   If  NOT BooExistPos405
     Then
       Exit ;
   k:=NumOrd405[NPlanet];
   l:=NumCof405[NPlanet];
   l0:=l-1;
   m:=NumDay405[NPlanet];
   w:=m;
   rw:=2/w;
   t:=TinTDB;
   Tn:=CoefEph405[1];
   Tk:=CoefEph405[2];
   r:=(t-Tn)/m;
   n:=Trunc(r-1.0e-10);
   np:=k+3*l*n-1;
   Tnn:=Tn+m*n;
   Tkk:=Tnn+m;
   x:=-1+(t-Tnn)*rw;
   xw:=2*x;
   for i:=1 to 3 do
     for j:=0 to l0 do
       begin
         np:=np+1;
         CoefQuant[i,j]:=CoefEph405[np];
       end;
   PolQuanT[0]:=1; { Chebyshev polinoms }
   PolQuanT[1]:=x; { initial values }
   PolQuanU[0]:=1;
   PolQuanU[1]:=xw;
   for i:=2 to l do
     Begin
       i1:=i-1; { Chebyshev polinoms }
       i2:=i-2; { current values }
       PolQuanT[i]:=xw*PolQuanT[i1]-PolQuanT[i2];
       PolQuanU[i]:=xw*PolQuanU[i1]-PolQuanU[i2];
     End;
   for i:=1 to 3 do
     Begin
       xr:=CoefQuant[i,0];
       xv:=0.0;
       for j:=1 to l0 do
         Begin
           xr:=xr+CoefQuant[i,j]*PolQuanT[j];
           xv:=xv+j*CoefQuant[i,j]*PolQuanU[j-1];
         End;
       Pos[i]:=xr;            { position in km }
       Vel[i]:=rw*xv/86400.0; { velosity in km }
     End;
 End;

 { to obtain value for factor mu in (km**3/s**2)/c**2 }

 Procedure FactMu405 ;
 Var
   n  : Byte ;
   c  : Extended ;
   z  : Extended ;
 Begin
   { from (AstrUnit**3/(Ephemeris Day)**2)*Solar Mass
           to (km**3/s**3)*Solar Mass }
   c:=(AstrUnit*Sqr(AstrUnit)/Sqr(86400.000))*Sqr(GaussConst);
   z:=c/Sqr(VelOfLight); { in km * Solar Mass , VelOfLight in km/s }
   For n:=1 To NumPlanets Do
     Begin
       FMPlanet[n]:=c/ValueMass405[n]; { in km^3/s^2 }
       FMPlaner[n]:=z/ValueMass405[n]; { in km with c^2 }
     End;
 End;

 { ToGetF405 is a procedure
   to define name of the file for DE200/LE200 ephemeris }

 Procedure ToGetF405 ; { all variables are external }
 Const
   numfile   = 1 ;
   namefile  : Array [1..numfile] of String =
             ( 'd:\ephjpl\ephbinep.405' ) ;
 Var
   num  : Byte ;
   boo  : Boolean ;
   ior  : Integer ;
 Begin
   BooExistEph405:=FALSE;
   BooExistPos405:=FALSE;
   num:=0;
   Repeat
     num:=num+1;
     Assign(CoeFile405,namefile[num]);
 {$I-}
     ReSet(CoeFile405);
 {$I+}
     ior:=IOResult;
     If  ior IN [ 0 , 5 ]
       Then
         BooExistEph405:=TRUE;
     If  ior = 5
       Then { file read only }
         Begin
           FileMode:=0;
           ReSet(CoeFile405); { file is opened for reading }
           FileMode:=2;
         End;
   Until  ( ( BooExistEph405 ) OR ( num = numfile ) ) ;
   If  NOT BooExistEph405
     Then { no ephemeris file DE405/LE405 }
       Exit;
   { file is opened DE405/LE405 }
   Seek(CoeFile405,0); { the first record }
   Read(CoeFile405,CoefEph405);
   ZInit405:=CoefEph405[1]; { initial julian day }
   ZStep405:=CoefEph405[2]-CoefEph405[1]; { step in day }
   MRec405:=FileSize(CoeFile405)-1; { maximum number of records }
   Seek(CoeFile405,MRec405);
   Read(CoeFile405,CoefEph405);
   ZLast405:=CoefEph405[2]; { finish julian day }
   NRec405:=-1;
   FactMu405 ; { to (km**3/s**2) and for relativity to (km**3/s**2)/c**2 }
 End;

 { to read file with the DE405 data }

 Procedure ToRead405( TinTDB  : Extended ) ;
 Var
   irec    : LongInt ;
   r       : Extended;
 Begin
   BooExistPos405:=FALSE; { previous }
   If  NOT BooExistEph405
     Then
       ToGetF405;
   If  NOT BooExistEph405
     Then { no ephemeris file DE405/LE405 }
       Exit;
   If  ( ( ZInit405 <= TinTDB ) AND ( TinTDB <= ZLast405 ) )
     Then
       Begin
         r:=(TinTDB-ZInit405)/ZStep405;
         irec:=Trunc(r);
         If  irec > MRec405
           Then
             irec:=MRec405;
         If  irec <> NRec405
           Then
             Begin
               Seek(CoeFile405,irec);
               Read(CoeFile405,CoefEph405);
               NRec405:=irec;
             End;
         BooExistPos405:=TRUE; { there are ephemeris data }
       End;
  { DE405/LE405 Ephemeris has been read in massive  CoefEph405(1018) }
 End;

 { EarthB405 is a procedure
   to obtain the position of the Earth reffered to the barycenter
      of the Solar system from DE405/LE405 Ephemeris }

 Procedure EarthB405 ( TinTDB : Extended ;
                       Var  PosEarth , VelEarth : TVect3 ) ;
 Var
   i  : Byte ;
   PosMoon,VelMoon,
   PosBarEM,VelBarEM : TVect3 ;
 Begin
   PosVel405(NMoon,TinTDB,PosMoon,VelMoon);
   If  NOT BooExistPos405
     Then { no ephemeris for this date TinTDB }
       Exit ;
   PosVel405(NEarth,TinTDB,PosBarEM,VelBarEM);
   For i:=1 To 3 Do
     Begin
       PosEarth[i]:=PosBarEM[i]-CoefMassR405*PosMoon[i];
       VelEarth[i]:=VelBarEM[i]-CoefMassR405*VelMoon[i];
     End;
 End;

 { MoonB405 is a procedure
   to obtain the position of the Moon reffered to the barycenter
      of the Solar system from DE405/LE405 Ephemeris }

 Procedure MoonB405 ( TinTDB : Extended ;
                      Var  PosMoon , VelMoon : TVect3 ) ;
 Var
   i  : Byte ;
   PosM,VelM,
   PosEM,VelEM : TVect3 ;
 Begin
   PosVel405(NMoon,TinTDB,PosM,VelM); { the Moon to the Earth }
   If  NOT BooExistPos405
     Then { no ephemeris for this date TinTDB }
       Exit ;
   PosVel405(NEarth,TinTDB,PosEM,VelEM); { the Earth-Moon }
   For i:=1 To 3 Do
     Begin
       PosMoon[i]:=PosEM[i]+(1-CoefMassR405)*PosM[i];
       VelMoon[i]:=VelEM[i]+(1-CoefMassR405)*VelM[i];
     End;
 End;

 Procedure EarthMoonB405 ( TinTDB  : Extended ;
                           Var  PosEarth , VelEarth  : TVect3 ;
                           Var  PosMoon  , VelMoon   : TVect3 ) ;
 Var
   i  : Byte ;
   PosM,VelM,
   PosEM,VelEM : TVect3 ;
 Begin
   PosVel405(NMoon,TinTDB,PosM,VelM); { the Moon to the Earth }
   If  NOT BooExistPos405
     Then { no ephemeris for this date TinTDB }
       Exit ;
   PosVel405(NEarth,TinTDB,PosEM,VelEM); { the Earth-Moon }
   For i:=1 To 3 Do
     Begin
       PosEarth[i]:=PosEM[i]-CoefMassR405*PosM[i];
       VelEarth[i]:=VelEM[i]-CoefMassR405*VelM[i];
       PosMoon[i]:=PosEM[i]+(1-CoefMassR405)*PosM[i];
       VelMoon[i]:=VelEM[i]+(1-CoefMassR405)*VelM[i];
     End;
 End;

 Procedure AccEarth405 ( t  : Extended ;  Var  a  : TVect3 ) ;
 Var
   i  : Byte ;
   d  : Extended ;
   s  : Extended ;
   p  : TVect3 ;
   u  : TVect3 ;
   v  : TVect3 ;
 Begin
   s:=+0.1; { in day }
   d:=t-s;
   EarthB405(d,p,u); { in km/s }
   d:=t+s;
   EarthB405(d,p,v); { in km/s }
   For i:=1 To 3 Do
     a[i]:=0.5*(v[i]-u[i])/(86400*s); { in km/s**2 }
 End;

 END.
