
 { in units
   UnConTyp  are types , constants
   Un2000_C  are simple procedures for arguments
             a value for the Earth Rotation Angle Theta
   Un2000_L  is procedure to calculate the Luni-Solar nutation
   Un2000_P  is procedure to calculate the planetary terms
   UnPN2000  are the procedures to obtain matrixes P(t) N(t)
   UnQR2000  are the procedures to obtain matrixes Q(t) R(t)
                 the position of the CIP in the GCRS
   Un2000_S  are the procedures to obtain
             the values for s(t) and GST , coefficents are in memory
   UnXY2000  are the procedures to obtain
             a value for the parameters X Y , coefficients are in memory
   UnZFiles  are the procedures to obtain
                 the values for the parameters X , Y
                 the values for s(t) and Greenwich Sidereal Time GMT
                 coefficients are read from the files
                 N2000_X.TXT , N2000_Y.TXT , N2000_S.TXT , N2000_3.TXT
   UnZS2000  are the procedures to compare different method of calculating
                 the position of the CEO on the equator of the CIP
   a try to calculate with MHB2000A precession/nutation theory
   Resolution B1.6
   of General Assembly of the International Astronomical Union 
   from  IERS Conventions 2000 , web site
   http:\\maia.usno.navy.mil\conv2000.html }

 UNIT UnXY2000 ;

 INTERFACE

 Uses
   UnConTyp ;

 { approximating value for X Y from the formulae }

 Procedure FormulaeXY  ( DateJulian  : Extended ;
                         Var   XCip  : Extended ;
                         Var   YCip  : Extended ) ;

 { more accurate value for X Y Z }

 Procedure ForPNPoleXY ( DateJulian  : Extended ;
                         Var  RPole  : TVect3 ) ;

 IMPLEMENTATION

 Uses
   UnPN2000 , { PNMatrix }
   Un2000_L , { lunisolar terms nutation }
   Un2000_P , { planets terms nutation }
   UnForFun ; { MultMatrVec }

 { to obtain X and Y position of the CIP just using simple formulae }

 Procedure FormulaeXY  ( DateJulian  : Extended ;
                         Var   XCip  : Extended ;
                         Var   YCip  : Extended ) ;
 Const
   dksi0  = -0.0166170 ; { offset on OX axes in arcsec }
   deta0  = -0.0068192 ; { offset on OY axes in arcsec }
   dalf0  = -0.0146000 ; { the R.A. of the mean equinox in arcsec }
 Var
   dplu,delu,dppl,depl  : Extended ;
   t,epso,psia,oma,xia,epsa  : Extended ;
   dpsi,dpsi1,deps,deps1,psi,om,x,y  : Extended ;
 Begin
   t:=(DateJulian-JD2000)/JulianC; { from the fundamental epoch in century }
   epso:=SecRad*84381.448; { in radian }
   psia:=SecRad*(5038.47875+(-1.07259-0.001147*t)*t)*t;
   oma:=epso+SecRad*(-0.02542+(0.05127-0.007726*t)*t)*t;
   xia:=SecRad*(10.5526+(-2.38064-0.001125*t)*t)*t;
   epsa:=epso+SecRad*(-46.84024+(-0.00059+0.001813*t)*t)*t;
   NutParmLuSo(DateJulian,dplu,delu); { Un2000_L }
   NutParmPlan(DateJulian,dppl,depl); { Un2000_P }
   dpsi:=SecRad*(dplu+dppl); { from arcsec to radian }
   deps:=SecRad*(delu+depl);
   dpsi1:=(dpsi*Sin(epsa)*Cos(xia)-deps*Sin(xia))/Sin(oma);
   deps1:=dpsi*Sin(epsa)*sin(xia)+deps*cos(xia);
   psi:=psia+dpsi1;
   om:=oma+deps1;
   x:=+Sin(om)*Sin(psi); { in radian }
   y:=-Sin(epso)*Cos(om)+Cos(epso)*Sin(om)*Cos(psi);
   XCip:=x+SecRad*(dksi0-dalf0*y); { in radian }
   YCip:=y+SecRad*(deta0+dalf0*x); { in radian }
 End;

 { to obtain the vector R(X,Y,Z)
   of the Celestial Intermediate Pole
   in the Geocentric Celestial Reference System
   by using the precession-nutation theory MHB2000 }

 Procedure ForPNPoleXY ( DateJulian  : Extended ;
                         Var  RPole  : TVect3 ) ;
 Const
   dksi0  = -0.0166170 ; { offset on OX axes in arcsec }
   deta0  = -0.0068192 ; { offset on OY axes in arcsec }
   dalf0  = -0.0146000 ; { the R.A. of the mean equinox in arcsec }
 Var
   vea,vep  : TVect3 ;
   pna,pnp  : TMatr33 ;
   x,y,z    : Extended ;
 Begin
   vea[1]:=0.0;
   vea[2]:=0.0;
   vea[3]:=1.0; { unit vector in the CIP system }
   ForPNMatrix(DateJulian,pna,pnp); { matrixes of precession-nutation }
   MultMatrVec(pnp,vea,vep); { unit vector in the GCRS but without offset }
   x:=vep[1]+SecRad*(dksi0-dalf0*vep[2]); { to add offset but }
   y:=vep[2]+SecRad*(deta0+dalf0*vep[1]); { to disturb unit vector }
   z:=Sqrt(1-(Sqr(x)+Sqr(y))); { for doing unit vector }
   RPole[1]:=x; { X=cos(E)*sin(d) }
   RPole[2]:=y; { Y=sin(E)*sin(d) }
   RPole[3]:=z; { Z=cos(d) }
 End;

 END.
