
 { in units
   Un2000_C  are types , constants and simple procedures for arguments
   Un2000_L  is procedure to calculate the Luni-Solar nutation
   Un2000_P  is procedure to calculate the planetary terms
   UnPN2000  are the procedures to obtain matrixes
   UnXY2000  are the procedures to obtain
                 the position of the CIP in the GCRS
   UnZS2000  are the procedures to compare different method of calculating
                 the position of the CEO on the equator of the CIP
   a try to calculate with MHB2000A precession/nutation theory
   Resolution B1.6
   of General Assembly of the International Astronomical Union 
   from  IERS Conventions 2000 , web site
   http:\\maia.usno.navy.mil\conv2000.html }

 UNIT UnZS2000 ;

 INTERFACE

 Procedure CEOPosStepByStep ( DateJulianMax  : Extended ) ;

 Procedure CEOPosFromFile ( DateJulianMax  : Extended ) ;

 Procedure CEOCompare ( DateJulianMax  : Extended ) ;

 IMPLEMENTATION

 Uses
   UnConTyp ,
   UnXY2000 , Un2000_S , UnZFiles ;

 Var
   fc  : Text ;

 { to get integral expression
   [(X(t)*d/dt(Y(t)-Y(t)*d/dt(X(t))/1+Z(t)] 
   with the use of numerical differentiation }

 Function ToGetExpres ( da , dt  : Extended ) : Extended ;
 Var
   rp     : TVect3 ;
   ds     : Extended ;
   x,y,z  : Extended ;
   x1,y1  : Extended ;
   x2,y2  : Extended ;
   vx,vy  : Extended ;
 Begin
   ForPNPoleXY(da,rp);
   x:=rp[1];
   y:=rp[2];
   z:=rp[3];
   ds:=da-dt; { one step earlier }
   ForPNPoleXY(ds,rp);
   x1:=rp[1];
   y1:=rp[2];
   ds:=da+dt; { one step later }
   ForPNPoleXY(ds,rp);
   x2:=rp[1];
   y2:=rp[2];
   vx:=0.5*(x2-x1)/dt; { d/dt(X(t)) }
   vy:=0.5*(y2-y1)/dt; { d/dt(Y(t)) }
   ToGetExpres:=(x*vy-y*vx)/(1+z); { in radian }
 End;

 { to get the numerical value of the s(t)
   that "being a quantity wich provides the position
   of the Celestial Ephemeris Origin
   on the equator of the Celestial Intermediate Pole"
   s(t)=-Int(t0,t)[(X(t)*d/dt(Y(t)-Y(t)*d/dt(X(t))/1+Z(t)]dt+s(t0) }

 Procedure CEOPosStepByStep ( DateJulianMax  : Extended ) ;
 Var
   ic  : Byte ;
   da  : Extended ;
   dm  : Extended ;
   dd  : Extended ;
   dt  : Extended ;
   ta  : Extended ;
   tc  : Extended ;
   s0  : Extended ;
   st  : Extended ;
   ra  : Extended ;
   rb  : Extended ;
 Begin
   Assign(fc,'ceo_posa.prt');
   ReWrite(fc);
   WriteLn(fc,'CEO position in arcsec by the use numerical integration');
   dm:=DateJulianMax;
   dd:=0.5; { step in day for numerical integration }
   dt:=0.05; { step in day for numerical differentiation }
   s0:=0.0; { initial }
   st:=s0;
   da:=JD2000; { from the standard epoch }
   ra:=ToGetExpres(da,dt); { initial value }
   ic:=0;
   While  da < dm  Do
     Begin
       da:=da+dd;
       rb:=ToGetExpres(da,dt); { the next value }
       st:=st-0.5*(ra+rb)*dd; { the integral over one step in radian }
       ra:=rb;
       ic:=ic+1;
       If  ic = 2
         Then
           Begin
             WriteLn('CEO position in arcsec',da:15:2,RadSec*st:25:10);
             WriteLn(fc,da:15:2,RadSec*st:25:10);
             ic:=0;
           End;
     End;
   Close(fc);
 End;

 { to calculate the value for s(t)
   by the use series that contained in the file N2000_S.TXT }

 Procedure CEOPosFromFile ( DateJulianMax  : Extended ) ;
 Var
   da  : Extended ;
   dm  : Extended ;
   dd  : Extended ;
   s0  : Extended ;
   st  : Extended ;
   sr  : Extended ;
   xp  : Extended ;
   yp  : Extended ;
 Begin
   dm:=DateJulianMax;
   Assign(fc,'ceo_posf.prt');
   ReWrite(fc);
   WriteLn(fc,'CEO position in arcsec with the use series from the file');
   da:=JD2000;
   dd:=0.5;
   s0:=ValueCEOPos(da); { in arcsec function from unit UnXY2000.pas }
   While  da < dm  Do
     Begin
       da:=da+dd;
       st:=ValueCEOPos(da); { in arcsec }
       FormulaeXY(da,xp,yp);
       sr:=ToGetCEOPos(da,xp,yp);
       WriteLn('CEO position in arcsec',da:15:2,st:20:10,(st-sr):20:10);
       WriteLn(fc,da:15:2,st:25:10,(st-sr):20:10);
     End;
   Close(fc);
 End;

 { to compare the result from two method
   numerical integration and series for function s(t) }

 Procedure CEOCompare ( DateJulianMax  : Extended ) ;
 Var
   ic  : Byte ;
   da  : Extended ;
   dm  : Extended ;
   dd  : Extended ;
   dt  : Extended ;
   ta  : Extended ;
   tc  : Extended ;
   s0  : Extended ;
   st  : Extended ;
   sr  : Extended ;
   sf  : Extended ;
   ra  : Extended ;
   rb  : Extended ;
 Begin
   Assign(fc,'ceo_posc.prt');
   ReWrite(fc);
   WriteLn(fc,'to compare');
   WriteLn(fc,'CEO position by the use numerical integration');
   WriteLn(fc,'CEO position by the use series from file N2000_S.TXT');
   WriteLn(fc,'Julian Date ,',
              ' CEO position in ArcSec , difference in ArcSec');
   dm:=DateJulianMax;
   dd:=0.5; { step in day for numerical integration }
   dt:=0.05; { step in day for numerical differentiation }
   st:=0.0; { initial for numerical integration }
   da:=JD2000; { from the standard epoch }
   ra:=ToGetExpres(da,dt); { initial value for expression over integral }
   s0:=ValueCEOPos(da); { initial from series }
   ic:=0;
   While  da < dm  Do
     Begin
       da:=da+dd;
       rb:=ToGetExpres(da,dt); { the next value }
       st:=st-0.5*(ra+rb)*dd; { the integral over one step in radian }
       ra:=rb; { for the next step of integration }
       ic:=ic+1;
       If  ic = 2
         Then
           Begin
             sr:=RadSec*st;
             sf:=ValueCEOPos(da)-s0; { in arcsec }
             WriteLn('CEO compare in arcsec',da:12:2,sr:15:8,(sf-sr):15:8);
             WriteLn(fc,da:12:2,sr:15:8,(sf-sr):15:8);
             ic:=0;
           End;
     End;
   Close(fc);
 End;

 END.